/// <reference types="node" />
/// <reference types="express" />
import 'reflect-metadata';
import * as express from 'express';
import { Server } from 'http';
import { ControllerDefinition } from './controller/ControllerDefinition';
import { ControllerDefinitionConstructor, GiuseppePlugin, ParameterDefinitionConstructor, RouteDefinitionConstructor, RouteModificatorConstructor } from './GiuseppePlugin';
import { GiuseppeRegistrar } from './GiuseppeRegistrar';
import { GiuseppeRoute } from './routes/GiuseppeRoute';
import { ReturnType } from './routes/ReturnType';
/**
 * Internal interface for route registering. Convenience objects.
 *
 * @export
 * @interface RouteRegisterInformation
 */
export interface RouteRegisterInformation {
    route: GiuseppeRoute;
    ctrl: Function;
    segments: number;
    wildcards: number;
    urlParams: number;
}
/**
 * Main entry class for giuseppe. Does contain the necessary methods to get the application running.
 * Does export the configuration and plugin system.
 *
 * @export
 * @class Giuseppe
 */
export declare class Giuseppe {
    /**
     * Giuseppes item registrar. Is used to register controllers, routes, parameters and all other things that
     * giuseppe contains. Can be used even when giuseppe is not instantiated yet.
     *
     * @static
     * @type {GiuseppeRegistrar}
     * @memberof Giuseppe
     */
    static readonly registrar: GiuseppeRegistrar;
    /**
     * The actual server instance of express once the application has started.
     *
     * @readonly
     * @type {(Server | undefined)}
     * @memberof Giuseppe
     */
    readonly server: Server | undefined;
    /**
     * The express application behind this instance of giuseppe. Someone might want to change the used express instance
     * before calling [start()]{@link Giuseppe#start()}. Also, on this propert you can add other things like
     * compression or body-parser.
     *
     * @type {express.Express}
     * @memberof Giuseppe
     */
    expressApp: express.Express;
    /**
     * The router instance that is used for this instance of giuseppe. Access it to add additional routes or even
     * switch the whole router.
     *
     * @type {express.Router}
     * @memberof Giuseppe
     */
    router: express.Router;
    protected plugins: GiuseppePlugin[];
    protected routes: {
        [id: string]: RouteRegisterInformation;
    };
    protected _server: Server | undefined;
    protected _returnTypes: ReturnType<any>[] | null;
    protected _pluginController: ControllerDefinitionConstructor[] | null;
    protected _pluginRoutes: RouteDefinitionConstructor[] | null;
    protected _pluginRouteModificators: RouteModificatorConstructor[] | null;
    protected _pluginParameters: ParameterDefinitionConstructor[] | null;
    /**
     * List of registered {@link ReturnType}.
     *
     * @readonly
     * @protected
     * @type {ReturnType<any>[]}
     * @memberof Giuseppe
     */
    protected readonly returnTypes: ReturnType<any>[];
    /**
     * List of registered {@link ControllerDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {ControllerDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    protected readonly pluginController: ControllerDefinitionConstructor[];
    /**
     * List of registered {@link RouteDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {RouteDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    protected readonly pluginRoutes: RouteDefinitionConstructor[];
    /**
     * List of registered {@link RouteModificatorConstructor}.
     *
     * @readonly
     * @protected
     * @type {RouteModificatorConstructor[]}
     * @memberof Giuseppe
     */
    protected readonly pluginRouteModificators: RouteModificatorConstructor[];
    /**
     * List of registered {@link ParameterDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {ParameterDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    protected readonly pluginParameters: ParameterDefinitionConstructor[];
    constructor();
    /**
     * Registers a given plugin into this giuseppe instance. Clears the internal caches when it does so.
     * Calls the initialize method on a plugin.
     *
     * @param {GiuseppePlugin} plugin
     * @returns {this}
     * @memberof Giuseppe
     */
    registerPlugin(plugin: GiuseppePlugin): this;
    /**
     * Fires up the express application within giuseppe. Gathers all registered controllers and routes and registers
     * them on the given [router]{@link Giuseppe#router}. After the router is configured, fires up the express
     * application with the given parameter.
     *
     * @param {number} [port=8080] The port of the web application (express.listen argument).
     * @param {string} [baseUrl=''] Base url that is preceeding all urls in the system.
     * @param {string} [hostname] Hostname that is passed to express.
     * @param {Function} [callback] Callback that is used in express when the system is listening and ready.
     * @memberof Giuseppe
     */
    start(port?: number, baseUrl?: string, hostname?: string, callback?: Function): void;
    /**
     * Closes the server of the application.
     *
     * @param {Function} [callback] Callback that is passed to the server.
     * @memberof Giuseppe
     */
    stop(callback?: Function): void;
    /**
     * Loads controllers from the actual process directory with a given globbing pattern. The start directory is always
     * process.cwd(). With globbing, you can exclude, match, find files that should be loaded.
     * More information here: [glob]{@link https://www.npmjs.com/package/glob#glob-primer}.
     *
     * Can be used when you don't want to load all controllers by hand.
     *
     * @param {string} globPattern
     * @returns {Promise<void>}
     * @memberof Giuseppe
     *
     * @example
     * // load all files in build directory
     *
     * const giuseppe = new Giuseppe();
     * giuseppe.loadControllers('\* \* /build/ \* \* /*.js'); // <-- without the spaces of course.
     */
    loadControllers(globPattern: string): Promise<void>;
    /**
     * Configures the actual instance of the express router. Creates the registered routes for the controllers in giuseppe
     * as the first step. After that, registers each route to the router and returns the router.
     *
     * @param {string} [baseUrl=''] Base url, that is preceeding all routes.
     * @returns {express.Router} The configured router.
     * @memberof Giuseppe
     */
    configureRouter(baseUrl?: string): express.Router;
    /**
     * Ultimatively creates the routes that are registered in the registrar of giuseppe. For each controller there
     * is the following procedure:
     *  1. Check if the controller definition is registered as a plugin
     *  2. Create all routes of the controller (call .createRoutes(baseUrl))
     *  3. For each create route:
     *     - Load the routes modificators
     *     - If there are none, add routes to the list and continue
     *     - If there are any, throw the routes at the modificators (can be multiple)
     *     - Add routes to the list
     *  4. Create {@link RouteRegisterInformation} for each route
     *
     * @protected
     * @param {string} baseUrl
     * @memberof Giuseppe
     */
    protected createRoutes(baseUrl: string): void;
    protected registerRoutes(): void;
    /**
     * Helper function that creates the wrapping function around a route for express. This wrapping function
     * ensures the right `this` context, does parse the actual param values and handles errors.
     *
     * The resulting function is then passed to the express router.
     *
     * @protected
     * @param {RouteRegisterInformation} routeInfo
     * @returns {express.RequestHandler}
     * @memberof Giuseppe
     */
    protected createRouteWrapper(routeInfo: RouteRegisterInformation): express.RequestHandler;
    /**
     * Check if a given controller, the routes of the controller, the modificators and parameters of the route are
     * registered within a plugin in giuseppe. If not, throw an exception.
     *
     * @protected
     * @throws {DefinitionNotRegisteredError}
     * @param {ControllerDefinition} controller
     * @returns {boolean}
     * @memberof Giuseppe
     */
    protected checkPluginRegistration(controller: ControllerDefinition): boolean;
}
