"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
require("reflect-metadata");
const express = require("express");
const glob = require("glob");
const path_1 = require("path");
const GiuseppeCorePlugin_1 = require("./core/GiuseppeCorePlugin");
const errors_1 = require("./errors");
const DuplicateRouteError_1 = require("./errors/DuplicateRouteError");
const GiuseppeRegistrar_1 = require("./GiuseppeRegistrar");
const ReturnTypeHandler_1 = require("./ReturnTypeHandler");
const RouteDefinition_1 = require("./routes/RouteDefinition");
const ControllerMetadata_1 = require("./utilities/ControllerMetadata");
/**
 * Score sort function for route register information. Calculates the sorting score based on segments, url params
 * and wildcards.
 *
 * @param {RouteRegisterInformation} route
 */
const routeScore = (route) => route.segments * 1000 - route.urlParams * 0.001 - route.wildcards;
/**
 * Main entry class for giuseppe. Does contain the necessary methods to get the application running.
 * Does export the configuration and plugin system.
 *
 * @export
 * @class Giuseppe
 */
class Giuseppe {
    constructor() {
        /**
         * The express application behind this instance of giuseppe. Someone might want to change the used express instance
         * before calling [start()]{@link Giuseppe#start()}. Also, on this propert you can add other things like
         * compression or body-parser.
         *
         * @type {express.Express}
         * @memberof Giuseppe
         */
        this.expressApp = express();
        /**
         * The router instance that is used for this instance of giuseppe. Access it to add additional routes or even
         * switch the whole router.
         *
         * @type {express.Router}
         * @memberof Giuseppe
         */
        this.router = express.Router();
        this.plugins = [];
        this.routes = {};
        this.registerPlugin(new GiuseppeCorePlugin_1.GiuseppeCorePlugin());
    }
    /**
     * The actual server instance of express once the application has started.
     *
     * @readonly
     * @type {(Server | undefined)}
     * @memberof Giuseppe
     */
    get server() {
        return this._server;
    }
    /**
     * List of registered {@link ReturnType}.
     *
     * @readonly
     * @protected
     * @type {ReturnType<any>[]}
     * @memberof Giuseppe
     */
    get returnTypes() {
        if (!this._returnTypes) {
            this._returnTypes = this.plugins
                .filter(p => !!p.returnTypeHandler)
                .reduce((all, cur) => all.concat(cur.returnTypeHandler), []);
        }
        return this._returnTypes;
    }
    /**
     * List of registered {@link ControllerDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {ControllerDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    get pluginController() {
        if (!this._pluginController) {
            this._pluginController = this.plugins
                .filter(p => !!p.controllerDefinitions)
                .reduce((all, cur) => all.concat(cur.controllerDefinitions), []);
        }
        return this._pluginController;
    }
    /**
     * List of registered {@link RouteDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {RouteDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    get pluginRoutes() {
        if (!this._pluginRoutes) {
            this._pluginRoutes = this.plugins
                .filter(p => !!p.routeDefinitions)
                .reduce((all, cur) => all.concat(cur.routeDefinitions), []);
        }
        return this._pluginRoutes;
    }
    /**
     * List of registered {@link RouteModificatorConstructor}.
     *
     * @readonly
     * @protected
     * @type {RouteModificatorConstructor[]}
     * @memberof Giuseppe
     */
    get pluginRouteModificators() {
        if (!this._pluginRouteModificators) {
            this._pluginRouteModificators = this.plugins
                .filter(p => !!p.routeModificators)
                .reduce((all, cur) => all.concat(cur.routeModificators), []);
        }
        return this._pluginRouteModificators;
    }
    /**
     * List of registered {@link ParameterDefinitionConstructor}.
     *
     * @readonly
     * @protected
     * @type {ParameterDefinitionConstructor[]}
     * @memberof Giuseppe
     */
    get pluginParameters() {
        if (!this._pluginParameters) {
            this._pluginParameters = this.plugins
                .filter(p => !!p.parameterDefinitions)
                .reduce((all, cur) => all.concat(cur.parameterDefinitions), []);
        }
        return this._pluginParameters;
    }
    /**
     * Registers a given plugin into this giuseppe instance. Clears the internal caches when it does so.
     * Calls the initialize method on a plugin.
     *
     * @param {GiuseppePlugin} plugin
     * @returns {this}
     * @memberof Giuseppe
     */
    registerPlugin(plugin) {
        if (this.plugins.find(o => o.name === plugin.name)) {
            throw new errors_1.DuplicatePluginError(plugin.name);
        }
        this._pluginController = null;
        this._pluginParameters = null;
        this._pluginRouteModificators = null;
        this._pluginRoutes = null;
        this._returnTypes = null;
        plugin.initialize(this);
        this.plugins.push(plugin);
        return this;
    }
    /**
     * Fires up the express application within giuseppe. Gathers all registered controllers and routes and registers
     * them on the given [router]{@link Giuseppe#router}. After the router is configured, fires up the express
     * application with the given parameter.
     *
     * @param {number} [port=8080] The port of the web application (express.listen argument).
     * @param {string} [baseUrl=''] Base url that is preceeding all urls in the system.
     * @param {string} [hostname] Hostname that is passed to express.
     * @param {Function} [callback] Callback that is used in express when the system is listening and ready.
     * @memberof Giuseppe
     */
    start(port = 8080, baseUrl = '', hostname, callback) {
        const router = this.configureRouter(baseUrl);
        this.expressApp.use(router);
        this._server = this.expressApp.listen.apply(this.expressApp, [port, hostname, callback].filter(Boolean));
    }
    /**
     * Closes the server of the application.
     *
     * @param {Function} [callback] Callback that is passed to the server.
     * @memberof Giuseppe
     */
    stop(callback) {
        if (this._server) {
            this._server.close(callback);
            delete this._server;
        }
    }
    /**
     * Loads controllers from the actual process directory with a given globbing pattern. The start directory is always
     * process.cwd(). With globbing, you can exclude, match, find files that should be loaded.
     * More information here: [glob]{@link https://www.npmjs.com/package/glob#glob-primer}.
     *
     * Can be used when you don't want to load all controllers by hand.
     *
     * @param {string} globPattern
     * @returns {Promise<void>}
     * @memberof Giuseppe
     *
     * @example
     * // load all files in build directory
     *
     * const giuseppe = new Giuseppe();
     * giuseppe.loadControllers('\* \* /build/ \* \* /*.js'); // <-- without the spaces of course.
     */
    loadControllers(globPattern) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                console.info(`Loading controller for the glob pattern "${globPattern}".`);
                const files = yield new Promise((resolve, reject) => {
                    glob(globPattern, (err, matches) => {
                        if (err) {
                            reject(err);
                            return;
                        }
                        resolve(matches);
                    });
                });
                for (const file of files) {
                    console.info(`Loading file '${file}'.`);
                    require(path_1.join(process.cwd(), file));
                }
            }
            catch (e) {
                console.error(`An error happend during loading of controllers`, {
                    globPattern,
                    err: e,
                });
            }
        });
    }
    /**
     * Configures the actual instance of the express router. Creates the registered routes for the controllers in giuseppe
     * as the first step. After that, registers each route to the router and returns the router.
     *
     * @param {string} [baseUrl=''] Base url, that is preceeding all routes.
     * @returns {express.Router} The configured router.
     * @memberof Giuseppe
     */
    configureRouter(baseUrl = '') {
        this.createRoutes(baseUrl);
        this.registerRoutes();
        return this.router;
    }
    /**
     * Ultimatively creates the routes that are registered in the registrar of giuseppe. For each controller there
     * is the following procedure:
     *  1. Check if the controller definition is registered as a plugin
     *  2. Create all routes of the controller (call .createRoutes(baseUrl))
     *  3. For each create route:
     *     - Load the routes modificators
     *     - If there are none, add routes to the list and continue
     *     - If there are any, throw the routes at the modificators (can be multiple)
     *     - Add routes to the list
     *  4. Create {@link RouteRegisterInformation} for each route
     *
     * @protected
     * @param {string} baseUrl
     * @memberof Giuseppe
     */
    createRoutes(baseUrl) {
        const url = baseUrl.startsWith('/') ? baseUrl.substring(1) : baseUrl;
        for (const ctrl of Giuseppe.registrar.controller) {
            this.checkPluginRegistration(ctrl);
            const meta = new ControllerMetadata_1.ControllerMetadata(ctrl.ctrlTarget.prototype);
            const routes = ctrl.createRoutes(url);
            let ctrlRoutes = [];
            for (const route of routes) {
                const mods = meta.modificators(route.name);
                if (!mods.length) {
                    ctrlRoutes.push(route);
                    continue;
                }
                let modifiedRoutes = [route];
                for (const mod of mods) {
                    modifiedRoutes = mod.modifyRoute(modifiedRoutes);
                }
                ctrlRoutes = ctrlRoutes.concat(modifiedRoutes);
            }
            for (const route of ctrlRoutes) {
                if (this.routes[route.id]) {
                    throw new DuplicateRouteError_1.DuplicateRouteError(route);
                }
                this.routes[route.id] = {
                    route,
                    segments: route.url.split('/').length,
                    wildcards: route.url.split('*').length - 1,
                    urlParams: route.url.split('/').filter(s => s.indexOf(':') >= 0).length,
                    ctrl: ctrl.ctrlTarget,
                };
            }
        }
    }
    registerRoutes() {
        Object.keys(this.routes)
            .map(k => this.routes[k])
            .sort((a, b) => routeScore(b) - routeScore(a))
            .forEach(r => this.router[RouteDefinition_1.HttpMethod[r.route.method]](`/${r.route.url}`, ...r.route.middlewares, this.createRouteWrapper(r)));
    }
    /**
     * Helper function that creates the wrapping function around a route for express. This wrapping function
     * ensures the right `this` context, does parse the actual param values and handles errors.
     *
     * The resulting function is then passed to the express router.
     *
     * @protected
     * @param {RouteRegisterInformation} routeInfo
     * @returns {express.RequestHandler}
     * @memberof Giuseppe
     */
    createRouteWrapper(routeInfo) {
        const meta = new ControllerMetadata_1.ControllerMetadata(routeInfo.ctrl.prototype);
        const params = meta.parameters(routeInfo.route.name);
        const returnTypeHandler = new ReturnTypeHandler_1.ReturnTypeHandler(this.returnTypes);
        const ctrlInstance = new routeInfo.ctrl();
        return (req, res) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const paramValues = [];
            try {
                for (const param of params) {
                    paramValues[param.index] = param.getValue(req, res);
                }
                let result = routeInfo.route.function.apply(ctrlInstance, paramValues);
                if (result instanceof Promise) {
                    result = yield result;
                }
                if (params.some(p => p.canHandleResponse)) {
                    return;
                }
                returnTypeHandler.handleValue(result, res);
            }
            catch (e) {
                meta.errorHandler().handleError(ctrlInstance, req, res, e);
            }
        });
    }
    /**
     * Check if a given controller, the routes of the controller, the modificators and parameters of the route are
     * registered within a plugin in giuseppe. If not, throw an exception.
     *
     * @protected
     * @throws {DefinitionNotRegisteredError}
     * @param {ControllerDefinition} controller
     * @returns {boolean}
     * @memberof Giuseppe
     */
    checkPluginRegistration(controller) {
        if (!this.pluginController.some(p => controller instanceof p)) {
            throw new errors_1.DefinitionNotRegisteredError(controller.constructor.name);
        }
        const meta = new ControllerMetadata_1.ControllerMetadata(controller.ctrlTarget.prototype);
        for (const route of meta.routes()) {
            if (!this.pluginRoutes.some(p => route instanceof p)) {
                throw new errors_1.DefinitionNotRegisteredError(route.constructor.name);
            }
            for (const mod of meta.modificators(route.name)) {
                if (!this.pluginRouteModificators.some(p => mod instanceof p)) {
                    throw new errors_1.DefinitionNotRegisteredError(mod.constructor.name);
                }
            }
            for (const param of meta.parameters(route.name)) {
                if (!this.pluginParameters.some(p => param instanceof p)) {
                    throw new errors_1.DefinitionNotRegisteredError(param.constructor.name);
                }
            }
        }
        return true;
    }
}
/**
 * Giuseppes item registrar. Is used to register controllers, routes, parameters and all other things that
 * giuseppe contains. Can be used even when giuseppe is not instantiated yet.
 *
 * @static
 * @type {GiuseppeRegistrar}
 * @memberof Giuseppe
 */
Giuseppe.registrar = new GiuseppeRegistrar_1.GiuseppeRegistrar();
exports.Giuseppe = Giuseppe;
