"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ControllerMetadata_1 = require("./utilities/ControllerMetadata");
/**
 * Helper class that is used to register giuseppe - stuff to the specific controllers.
 *
 * @export
 * @class GiuseppeRegistrar
 */
class GiuseppeRegistrar {
    constructor() {
        /**
         * List of controller definitions.
         *
         * @type {ControllerDefinition[]}
         * @memberof GiuseppeRegistrar
         */
        this.controller = [];
    }
    /**
     * Registeres a controller definition within the register (list of controllers).
     *
     * @param {ControllerDefinition} controller
     * @memberof GiuseppeRegistrar
     */
    registerController(controller) {
        this.controller.push(controller);
    }
    /**
     * Register a route to a controllers metadata.
     *
     * @param {Object} controller
     * @param {RouteDefinition} route
     * @memberof GiuseppeRegistrar
     */
    registerRoute(controller, route) {
        const meta = new ControllerMetadata_1.ControllerMetadata(controller);
        meta.routes().push(route);
    }
    /**
     * Register a modificator for a route of a controller.
     *
     * @param {Object} controller
     * @param {string} routeName
     * @param {RouteModificator} modificator
     * @memberof GiuseppeRegistrar
     */
    registerRouteModificator(controller, routeName, modificator) {
        const meta = new ControllerMetadata_1.ControllerMetadata(controller);
        meta.modificators(routeName).push(modificator);
    }
    /**
     * Register a parameter for a route of a controller.
     *
     * @param {Object} controller
     * @param {string} routeName
     * @param {ParameterDefinition} parameter
     * @memberof GiuseppeRegistrar
     */
    registerParameter(controller, routeName, parameter) {
        const meta = new ControllerMetadata_1.ControllerMetadata(controller);
        meta.parameters(routeName).push(parameter);
    }
    /**
     * Register an error handler for a given error type for a controller.
     *
     * @param {Object} controller
     * @param {ErrorHandlerFunction<Error>} handler
     * @param {Function[]} errors
     * @memberof GiuseppeRegistrar
     */
    registerErrorHandler(controller, handler, errors) {
        const meta = new ControllerMetadata_1.ControllerMetadata(controller);
        for (const error of errors) {
            meta.errorHandler().addHandler(handler, error);
        }
    }
}
exports.GiuseppeRegistrar = GiuseppeRegistrar;
