/// <reference types="express" />
import 'reflect-metadata';
import { RequestHandler } from 'express';
import { ControllerDefinition } from '../../controller/ControllerDefinition';
import { GiuseppeRoute } from '../../routes/GiuseppeRoute';
/**
 * Controller decorator. Creates a {@link GiuseppeApiController} that is registered within {@link Giuseppe}.
 * All controllers are later used to generate their routes.
 *
 * @export
 * @param {(string | RequestHandler)} [routePrefixOrMiddleware] It's either a middleware (if no route prefix is used) or a
 *                                                              string that contains the routeprefix for this controller.
 * @param {...RequestHandler[]} middlewares The rest of the middlewares that should be called before all routes.
 * @returns {ClassDecorator}
 *
 * @example
 * // Without any params.
 * @Controller()
 * class FoobarCtrl {}
 *
 * @example
 * // With a route prefix and 1 middleware.
 * @Controller('routePrefix', (req, res, next) => next())
 * class FoobarCtrl {}
 *
 * @example
 * // Only with middlewares.
 * @Controller((req, res, next) => next(), (req, res, next) => next())
 * class FoobarCtrl {}
 */
export declare function Controller(routePrefixOrMiddleware?: string | RequestHandler, ...middlewares: RequestHandler[]): ClassDecorator;
/**
 * Default core controller of giuseppe. Contains the routes and generates them on "configureRouter()" of giuseppe.
 *
 * @export
 * @class GiuseppeApiController
 * @implements {ControllerDefinition}
 */
export declare class GiuseppeApiController implements ControllerDefinition {
    readonly ctrlTarget: Function;
    readonly routePrefix: string;
    readonly middlewares: RequestHandler[];
    constructor(ctrlTarget: Function, routePrefix?: string, middlewares?: RequestHandler[]);
    createRoutes(baseUrl: string): GiuseppeRoute[];
}
