"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const Giuseppe_1 = require("../../Giuseppe");
const ControllerMetadata_1 = require("../../utilities/ControllerMetadata");
const UrlHelper_1 = require("../../utilities/UrlHelper");
/**
 * Controller decorator. Creates a {@link GiuseppeApiController} that is registered within {@link Giuseppe}.
 * All controllers are later used to generate their routes.
 *
 * @export
 * @param {(string | RequestHandler)} [routePrefixOrMiddleware] It's either a middleware (if no route prefix is used) or a
 *                                                              string that contains the routeprefix for this controller.
 * @param {...RequestHandler[]} middlewares The rest of the middlewares that should be called before all routes.
 * @returns {ClassDecorator}
 *
 * @example
 * // Without any params.
 * @Controller()
 * class FoobarCtrl {}
 *
 * @example
 * // With a route prefix and 1 middleware.
 * @Controller('routePrefix', (req, res, next) => next())
 * class FoobarCtrl {}
 *
 * @example
 * // Only with middlewares.
 * @Controller((req, res, next) => next(), (req, res, next) => next())
 * class FoobarCtrl {}
 */
function Controller(routePrefixOrMiddleware, ...middlewares) {
    const routePrefix = routePrefixOrMiddleware && typeof routePrefixOrMiddleware === 'string' ?
        routePrefixOrMiddleware :
        '';
    if (routePrefixOrMiddleware && typeof routePrefixOrMiddleware === 'function') {
        middlewares.unshift(routePrefixOrMiddleware);
    }
    return (ctrl) => Giuseppe_1.Giuseppe.registrar.registerController(new GiuseppeApiController(ctrl, routePrefix, middlewares));
}
exports.Controller = Controller;
/**
 * Default core controller of giuseppe. Contains the routes and generates them on "configureRouter()" of giuseppe.
 *
 * @export
 * @class GiuseppeApiController
 * @implements {ControllerDefinition}
 */
class GiuseppeApiController {
    constructor(ctrlTarget, routePrefix = '', middlewares = []) {
        this.ctrlTarget = ctrlTarget;
        this.routePrefix = routePrefix;
        this.middlewares = middlewares;
    }
    createRoutes(baseUrl) {
        const meta = new ControllerMetadata_1.ControllerMetadata(this.ctrlTarget.prototype);
        const url = UrlHelper_1.UrlHelper.buildUrl(baseUrl, this.routePrefix);
        return meta.routes()
            .reduce((all, cur) => all.concat(cur.createRoutes(meta, url, this.middlewares)), []);
    }
}
exports.GiuseppeApiController = GiuseppeApiController;
