"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const errors_1 = require("../../errors");
/**
 * Declaration of primitive javascript types.
 *
 * @type {Function[]}
 */
const PRIMITIVE_TYPES = [Object, String, Array, Number, Boolean];
/**
 * Giuseppe base parameter class. Is used for all core parameters (like cookie or body) to generalize
 * some logic.
 *
 * @export
 * @abstract
 * @class GiuseppeBaseParameter
 * @implements {ParameterDefinition}
 */
class GiuseppeBaseParameter {
    constructor(name, type, index, required, validator, factory) {
        this.name = name;
        this.type = type;
        this.index = index;
        this.required = required;
        this.validator = validator;
        this.factory = factory;
        this.canHandleResponse = false;
    }
    getValue(request) {
        let value = this.getRawValue(request);
        value = this.parseValue(value);
        this.validateValue(value);
        return value;
    }
    parseValue(raw) {
        if (!raw) {
            return raw;
        }
        let factory;
        if (this.factory) {
            factory = this.factory;
        }
        else {
            factory = rawValue => {
                const ctor = this.type;
                if (rawValue.constructor === ctor) {
                    return rawValue;
                }
                return PRIMITIVE_TYPES.indexOf(ctor) !== -1 ? ctor(rawValue) : new ctor(rawValue);
            };
        }
        try {
            return factory(raw);
        }
        catch (e) {
            throw new errors_1.ParameterParseError(this.name, e);
        }
    }
    validateValue(parsed) {
        if ((parsed === null || parsed === undefined) && this.required) {
            throw new errors_1.RequiredParameterNotProvidedError(this.name);
        }
        if (!this.validator || (!this.required && (parsed === undefined || parsed === null))) {
            return;
        }
        const isValid = value => {
            const predicates = this.validator;
            if (Array.isArray(predicates)) {
                return predicates.every(p => p(value));
            }
            return predicates(value);
        };
        try {
            if (!isValid(parsed)) {
                throw new errors_1.ParameterValidationFailedError(this.name);
            }
        }
        catch (e) {
            if (e instanceof errors_1.ParameterValidationFailedError) {
                throw e;
            }
            let error;
            if (!(e instanceof Error)) {
                error = new Error(e);
            }
            else {
                error = e;
            }
            throw new errors_1.ParameterValidationFailedError(this.name, error);
        }
    }
}
exports.GiuseppeBaseParameter = GiuseppeBaseParameter;
