/**
 * Type for the parameter factory function.
 */
export declare type ParameterFactory<T> = (raw: any) => T;
/**
 * Type for a paramater validator (can be multiple).
 */
export declare type ParameterValidator = Validator | Validator[];
/**
 * Predicate for the given parameter. This validator is executed if set and if the return value is false, an error is thrown.
 *
 * @param {any} value - The parsed value of the parameter.
 * @returns {boolean} - True when the validation was successful, otherwise false.
 */
export declare type Validator = (value: any) => boolean;
/**
 * Function that creates a string validator with some properties.
 *
 * @param {boolean} [allowEmpty=false] - Let empty string "" be a valid result.
 * @param {number} [min] - Minimum length of the string.
 * @param {number} [max] - Maximum length of the string.
 * @returns {Validator} - Validator function for the given parameters.
 */
export declare function isString({allowEmpty, min, max}?: {
    allowEmpty?: boolean;
    min?: number;
    max?: number;
}): Validator;
/**
 * Function that creates a number validator with some properties.
 *
 * @param {number} [min] - Lowest possible value.
 * @param {number} [max] - Highest possible value.
 * @param {number} [multipleOf] - The value must be a multiple of this (i.e. value mod multipleOf === 0).
 * @returns {Validator} - Validator function for the given parameters.
 */
export declare function isNumber({min, max, multipleOf}?: {
    min?: number;
    max?: number;
    multipleOf?: number;
}): Validator;
/**
 * Function that creates an array validator with some properties.
 *
 * @param {number} [min] - Minimum items in array.
 * @param {number} [max] - Maximum items in array.
 * @param {Function|Function[]} [type] - The constructor function(s) (i.e. the type(s)) of the items in the array.
 * @param {Validator|Validator[]} [validator] - One or multiple (array) validators for the array items.
 *                                              If multiple are provided, they are logically ANDed.
 * @returns {Validator} - Validator function for the given parameters.
 */
export declare function isArray({min, max, type, validator}?: {
    min?: number;
    max?: number;
    type?: Function | Function[];
    validator?: Validator | Validator[];
}): Validator;
