/// <reference types="express" />
import 'reflect-metadata';
import { RequestHandler } from 'express';
import { GiuseppeRoute } from '../../routes/GiuseppeRoute';
import { HttpMethod, RouteDefinition } from '../../routes/RouteDefinition';
import { ControllerMetadata } from '../../utilities/ControllerMetadata';
/**
 * Type for giuseppes route decorators.
 * Does generically restrain the MethodDecorator to Functions only.
 */
export declare type FunctionMethodDecorator = (target: Object, _: string | symbol, descriptor: TypedPropertyDescriptor<Function>) => void;
/**
 * Route decorator. Creates a route definition that reacts to a specified request. The method needs to be specified.
 * Can define one or multiple middlewares that are registered for that route. Also, a route name can be specified
 * that creates the url for express.
 *
 * @export
 * @param {HttpMethod} method The http method to use.
 * @param {(string | RequestHandler)} [routeOrMiddleware] Either a string that represents the url for this route, or
 *                                                        an optional middleware if no specific route is needed.
 * @param {...RequestHandler[]} middlewares Other middlewares that are used for this route.
 * @returns {FunctionMethodDecorator}
 */
export declare function Route(method: HttpMethod, routeOrMiddleware?: string | RequestHandler, ...middlewares: RequestHandler[]): FunctionMethodDecorator;
/**
 * Default core base route of giuseppe. Is configurable with all defined http methods of express (from the source).
 * Specifies it's http method via the enum {@link HttpMethod}.
 *
 * @export
 * @class GiuseppeBaseRoute
 * @implements {RouteDefinition}
 */
export declare class GiuseppeBaseRoute implements RouteDefinition {
    readonly httpMethod: HttpMethod;
    readonly routeFunction: Function;
    readonly route: string;
    readonly middlewares: RequestHandler[];
    readonly name: string;
    constructor(httpMethod: HttpMethod, routeFunction: Function, route?: string, middlewares?: RequestHandler[]);
    createRoutes(_: ControllerMetadata, baseUrl: string, controllerMiddlewares: RequestHandler[]): GiuseppeRoute[];
}
