"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const __1 = require("../..");
const RouteDefinition_1 = require("../../routes/RouteDefinition");
const UrlHelper_1 = require("../../utilities/UrlHelper");
/**
 * Route decorator. Creates a route definition that reacts to a specified request. The method needs to be specified.
 * Can define one or multiple middlewares that are registered for that route. Also, a route name can be specified
 * that creates the url for express.
 *
 * @export
 * @param {HttpMethod} method The http method to use.
 * @param {(string | RequestHandler)} [routeOrMiddleware] Either a string that represents the url for this route, or
 *                                                        an optional middleware if no specific route is needed.
 * @param {...RequestHandler[]} middlewares Other middlewares that are used for this route.
 * @returns {FunctionMethodDecorator}
 */
function Route(method, routeOrMiddleware, ...middlewares) {
    const route = routeOrMiddleware && typeof routeOrMiddleware === 'string' ? routeOrMiddleware : '';
    if (routeOrMiddleware && typeof routeOrMiddleware === 'function') {
        middlewares.unshift(routeOrMiddleware);
    }
    return (target, _, descriptor) => {
        if (!descriptor.value) {
            throw new TypeError(`Function is undefined in route ${route}`);
        }
        __1.Giuseppe.registrar.registerRoute(target, new GiuseppeBaseRoute(method, descriptor.value, route, middlewares));
    };
}
exports.Route = Route;
/**
 * Default core base route of giuseppe. Is configurable with all defined http methods of express (from the source).
 * Specifies it's http method via the enum {@link HttpMethod}.
 *
 * @export
 * @class GiuseppeBaseRoute
 * @implements {RouteDefinition}
 */
class GiuseppeBaseRoute {
    constructor(httpMethod, routeFunction, route = '', middlewares = []) {
        this.httpMethod = httpMethod;
        this.routeFunction = routeFunction;
        this.route = route;
        this.middlewares = middlewares;
    }
    get name() {
        return this.routeFunction.name;
    }
    createRoutes(_, baseUrl, controllerMiddlewares) {
        return [
            {
                id: `${RouteDefinition_1.HttpMethod[this.httpMethod]}_${UrlHelper_1.UrlHelper.buildUrl(baseUrl, this.route)}`,
                name: this.name,
                method: this.httpMethod,
                url: UrlHelper_1.UrlHelper.buildUrl(baseUrl, this.route),
                middlewares: [...controllerMiddlewares, ...this.middlewares],
                function: this.routeFunction,
            },
        ];
    }
}
exports.GiuseppeBaseRoute = GiuseppeBaseRoute;
