/// <reference types="express" />
import 'reflect-metadata';
import { Request, Response } from 'express';
/**
 * Default error handler. Does console.error with the error and sets the
 * http response code to 500.
 *
 * @export
 * @param {Request} _ express request object
 * @param {Response} res express response object
 * @param {Error} err error that happend
 */
export declare const DEFAULT_ERROR_HANDLER: (_: any, res: any, err: any) => void;
/**
 * @typedef ErrorHandlerFunction
 * @export
 * @type {Function}
 * @param {Request} request ExpressJS request object.
 * @param {Response} response ExpressJS response object.
 * @param {Error} error The error that happend.
 */
export declare type ErrorHandlerFunction<T extends Error> = (request: Request, response: Response, error: T) => void;
/**
 * Manager object that handles all ErrorHandlerFunctions for a controller. Registers itself in the controllers
 * metadata. If an error is thrown, just pass it to the error handler. The handler will do all the magic.
 *
 * @export
 * @class
 */
export declare class ControllerErrorHandler {
    private handlers;
    /**
     * Adds an error handler for the current controller with the given errorType. If the errorType is omitted,
     * the handler registers the function as a 'default' error handler.
     *
     * @param {ErrorHandlerFunction} handler The error handler to register.
     * @param {Error} [errorType] Error class that should be registered. If omitted, the handler is registered as default.
     */
    addHandler<T extends Error>(handler: ErrorHandlerFunction<T>, errorType?: Function): void;
    /**
     * Calls the error handler for a given error.
     *
     * @param {any} context Context of the error handler.
     * @param {Request} request ExpressJS request object.
     * @param {Response} response ExpressJS response object.
     * @param {Error} error Error object that was thrown.
     */
    handleError<T extends Error>(context: any, request: Request, response: Response, error: T): void;
}
