/// <reference types="express" />
import { RequestHandler } from 'express';
import { ControllerMetadata } from '../utilities/ControllerMetadata';
import { GiuseppeRoute } from './GiuseppeRoute';
/**
 * Reflect metadata key for route definitions.
 *
 * @type {string}
 */
export declare const ROUTE_DEFINITION_KEY = "giuseppe:RouteDefintion";
/**
 * Enum with all supported http methods from express. Full list is available here:
 * {@link https://expressjs.com/en/api.html#routing-methods|routing-methods}
 *
 * @export
 * @enum {number}
 */
export declare enum HttpMethod {
    get = 0,
    put = 1,
    post = 2,
    delete = 3,
    head = 4,
    connect = 5,
    options = 6,
    trace = 7,
    copy = 8,
    lock = 9,
    mkcol = 10,
    move = 11,
    propfind = 12,
    proppatch = 13,
    search = 14,
    unlock = 15,
    report = 16,
    mkactivity = 17,
    checkout = 18,
    merge = 19,
    msearch = 20,
    notify = 21,
    subscribe = 22,
    unsubscribe = 23,
    patch = 24,
    purge = 25,
}
/**
 * A route definition contains all relevant information for a route in a controller. This definition must be registered
 * in a giuseppe plugin. Those routes should be called by the controller and create a route (or a set of routes) that
 * are later registered within express.
 *
 * @export
 * @interface RouteDefinition
 */
export interface RouteDefinition {
    /**
     * Route url. Is sanitized, concatenated with controller and base prefixes and later registered in express.
     *
     * @type {string}
     * @memberof RouteDefinition
     */
    readonly route: string;
    /**
     * Used {@link HttpMethod} for this route.
     *
     * @type {HttpMethod}
     * @memberof RouteDefinition
     */
    readonly httpMethod: HttpMethod;
    /**
     * Function that is associated with this route.
     *
     * @type {Function}
     * @memberof RouteDefinition
     */
    readonly routeFunction: Function;
    /**
     * Name for the route.
     *
     * @type {string}
     * @memberof RouteDefinition
     */
    readonly name: string;
    /**
     * A list of middlewares that preceed this route. Those middlewares are concatenated with the ones of
     * the containing controller.
     *
     * @type {RequestHandler[]}
     * @memberof RouteDefinition
     */
    readonly middlewares: RequestHandler[];
    /**
     * Create a list of routes out of this definition. A route can create multiple giuseppe routes. The same rules
     * apply to all created routes: no duplicate IDs.
     *
     * @param {ControllerMetadata} meta
     * @param {string} baseUrl
     * @param {RequestHandler[]} middlewares
     * @returns {GiuseppeRoute[]}
     * @memberof RouteDefinition
     */
    createRoutes(meta: ControllerMetadata, baseUrl: string, middlewares: RequestHandler[]): GiuseppeRoute[];
}
