import 'reflect-metadata';
import { ControllerErrorHandler } from '../errors/ControllerErrorHandler';
import { ParameterDefinition } from '../parameter/ParameterDefinition';
import { RouteDefinition } from '../routes/RouteDefinition';
import { RouteModificator } from '../routes/RouteModificator';
/**
 * Helper class that manages metadata elements on a controller class.
 *
 * @export
 * @class ControllerMetadata
 */
export declare class ControllerMetadata {
    private controller;
    constructor(controller: Function | Object);
    /**
     * Returns the defined routes of a controller class. If no routes are defined, defines the array of routes
     * on the controller and returns the empty array.
     *
     * @returns {RouteDefinition[]}
     * @memberof ControllerMetadata
     */
    routes(): RouteDefinition[];
    /**
     * Returns the defined route modificators for a given route name of a controller class.
     * If no modificators are defined, defines the array of modificators on the controller and returns the empty array.
     *
     * @param {string} name
     * @returns {RouteModificator[]}
     * @memberof ControllerMetadata
     */
    modificators(name: string): RouteModificator[];
    /**
     * Returns the defined route parameters for a given route name of a controller class.
     * If no parameters are defined, defines the array of parameters on the controller and returns the empty array.
     *
     * @param {string} name
     * @returns {ParameterDefinition[]}
     * @memberof ControllerMetadata
     */
    parameters(name: string): ParameterDefinition[];
    /**
     * Returns the types for the parameters of a given route name. Returns an empty array if the route has no parameters.
     *
     * @param {string} name
     * @returns {Function[]}
     * @memberof ControllerMetadata
     */
    parameterTypes(name: string): Function[];
    /**
     * Returns the defined error handler of a controller class. If no handler is defined, defines the empty handler
     * on the controller and returns it.
     *
     * @returns {ControllerErrorHandler}
     * @memberof ControllerMetadata
     */
    errorHandler(): ControllerErrorHandler;
}
