"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const ControllerErrorHandler_1 = require("../errors/ControllerErrorHandler");
const ErrorHandlerDecorator_1 = require("../errors/ErrorHandlerDecorator");
const ParameterDefinition_1 = require("../parameter/ParameterDefinition");
const RouteDefinition_1 = require("../routes/RouteDefinition");
const RouteModificator_1 = require("../routes/RouteModificator");
/**
 * Helper class that manages metadata elements on a controller class.
 *
 * @export
 * @class ControllerMetadata
 */
class ControllerMetadata {
    constructor(controller) {
        this.controller = controller;
    }
    /**
     * Returns the defined routes of a controller class. If no routes are defined, defines the array of routes
     * on the controller and returns the empty array.
     *
     * @returns {RouteDefinition[]}
     * @memberof ControllerMetadata
     */
    routes() {
        if (!Reflect.hasOwnMetadata(RouteDefinition_1.ROUTE_DEFINITION_KEY, this.controller)) {
            Reflect.defineMetadata(RouteDefinition_1.ROUTE_DEFINITION_KEY, [], this.controller);
        }
        return Reflect.getOwnMetadata(RouteDefinition_1.ROUTE_DEFINITION_KEY, this.controller);
    }
    /**
     * Returns the defined route modificators for a given route name of a controller class.
     * If no modificators are defined, defines the array of modificators on the controller and returns the empty array.
     *
     * @param {string} name
     * @returns {RouteModificator[]}
     * @memberof ControllerMetadata
     */
    modificators(name) {
        if (!Reflect.hasOwnMetadata(RouteModificator_1.ROUTE_MODIFICATOR_KEY, this.controller, name)) {
            Reflect.defineMetadata(RouteModificator_1.ROUTE_MODIFICATOR_KEY, [], this.controller, name);
        }
        return Reflect.getOwnMetadata(RouteModificator_1.ROUTE_MODIFICATOR_KEY, this.controller, name);
    }
    /**
     * Returns the defined route parameters for a given route name of a controller class.
     * If no parameters are defined, defines the array of parameters on the controller and returns the empty array.
     *
     * @param {string} name
     * @returns {ParameterDefinition[]}
     * @memberof ControllerMetadata
     */
    parameters(name) {
        if (!Reflect.hasOwnMetadata(ParameterDefinition_1.PARAMETER_DEFINITION_KEY, this.controller, name)) {
            Reflect.defineMetadata(ParameterDefinition_1.PARAMETER_DEFINITION_KEY, [], this.controller, name);
        }
        return Reflect.getOwnMetadata(ParameterDefinition_1.PARAMETER_DEFINITION_KEY, this.controller, name);
    }
    /**
     * Returns the types for the parameters of a given route name. Returns an empty array if the route has no parameters.
     *
     * @param {string} name
     * @returns {Function[]}
     * @memberof ControllerMetadata
     */
    parameterTypes(name) {
        return Reflect.getOwnMetadata('design:paramtypes', this.controller, name) || [];
    }
    /**
     * Returns the defined error handler of a controller class. If no handler is defined, defines the empty handler
     * on the controller and returns it.
     *
     * @returns {ControllerErrorHandler}
     * @memberof ControllerMetadata
     */
    errorHandler() {
        if (!Reflect.hasOwnMetadata(ErrorHandlerDecorator_1.ERRORHANDLER_KEY, this.controller)) {
            Reflect.defineMetadata(ErrorHandlerDecorator_1.ERRORHANDLER_KEY, new ControllerErrorHandler_1.ControllerErrorHandler(), this.controller);
        }
        return Reflect.getOwnMetadata(ErrorHandlerDecorator_1.ERRORHANDLER_KEY, this.controller);
    }
}
exports.ControllerMetadata = ControllerMetadata;
